package Resources;
import java.util.ArrayList;
import java.util.Random;

@SuppressWarnings("unchecked") // Don't worry about this line

// People are just like MobileObjects, except they
// can hold possessions, and tend to move of their
// own free will based on their restlessness value

public class Person extends MobileObject {
	ArrayList<OwnableObject> possessions; // A list of the Person's belongings
	// How inclined a person is to move in a given time step.
	// If set to 1.0, the person will always move.  If set to 0.0,
	// the person never moves, and if set to 0.3, the person will
	// move 3 times out of 10
	double restlessness;
	
	// The constructor for the Person class
	public Person(String name) {
		// As usual, we start with the constructor for
		// the superclass
		super(name);
		// Initialize the possessions list so that it
		// starts with no items
		possessions = new ArrayList<OwnableObject>();
		// A person's restlessness is 0 by default
		restlessness = 0.0;
	}
	
	// Return a copy of the list of the Person's possessions.
	// Why a copy? We may want to perform calculations on the
	// possession list once we have it. If we didn't return a copy,
	// any changes would affect the original list
	public ArrayList<SimObject> get_possessions() {
		return (ArrayList<SimObject>) possessions.clone();
	}
	
	// Tell the Person to declare what possessions they have
	public void display_possessions() {
		say("I have " + possessions.toString());
	}
	
	// Have the Person say a message. If they are installed
	// somewhere, they will first note where they are saying
	// the message
	public void say(String utterance) {
		if (is_installed()) {
			System.out.print("At " + location.name + ": ");
		}
		System.out.println(name + " says --- " + utterance);
	}
	
	// Have the Person freak out 
	public void have_fit() {
		say("Yaaaah! I am upset!");
	}
	
	// Have the Person look around. The person will report
	// what things are at the same Place, as well as what
	// direction the person can move in from here
	public void look() {
		// A Person cannot look until given put at 
		// a specific location
		if (!is_installed()) {
			note("Cannot look until person is installed");
			return;
		}
		// Get a list of the things at the Person's location
		ArrayList<PhysicalObject> otherthings = location.get_things();
		
		// Remove the Person from this list--We don't want a 
		// Person to see him/herself
		otherthings.remove(this);	
		
		// We will use this list of things to build a nicely
		// formatted output message detailing the things
		// at the Person's location
		String str = "";
		
		// We only need to print out a list of things if 
		// the list actually has some things in it
		if (otherthings.size() > 0) {
			
			// For each of the elements in our list, add the
			// name of the thing to our new output string.
			// We also add a comma to make it pretty
			for (int i=0; i<otherthings.size()-1; i++) {
				str = str + otherthings.get(i).name + ", ";
			}
			
			// The last element in the list gets no comma
			str = str + otherthings.get(otherthings.size()-1).name;
			
			// Print out the string we just constructed
			say("I see " + str);
		}
		else {
			say("I don't see anything.");
		}
		
		// Repeat what we just did with the things at a location
		// with the possible exits from that location
		str = "";
		ArrayList<String> exits = location.get_exits();
		if (exits.size() > 0) {
			for (int i=0; i<exits.size()-1; i++) {
				str = str + exits.get(i).toString() + ", ";
			}
			str = str + exits.get(exits.size()-1);
			say("I can go " + str);
		}
		else {
			say("I can't go anywhere.");
		}
	}
	
	// Take is used for a Person to grab an OwnableObject
	// at the same place as the Person and make it one of
	// his/her possessions
	public void take(PhysicalObject thing) {
		
		// A Person must be at a location in order to take anything
		if (!is_installed()) {
			note("Cannot take until person is installed.");
			return;
		}
		
		// A Person cannot take something he/she already has
		else if (possessions.contains(thing)) {
			say("I already have " + thing.toString());
			return;
		}
		
		// Check to make sure the thing in question meets
		// all of the relevant criteria. The object must
		// first actually be at the same location as the 
		// Person
		else if (location.get_things().contains(thing)) {
			
			// The object must be an OwnableObject
			if (thing.is_ownable()) {
				
				// Now that we know the object is an OwnableObject
				// we CAST it to be of type OwnableObject so we
				// can call OwnableObject methods. Without a cast,
				// we would only be able to use methods that 
				// all PhysicalObjects have, since thing is of
				// type PhysicalObject
				OwnableObject ownable_thing = (OwnableObject) thing;
				
				// If someone else owns the object, we take
				// it away from that Person
				if (ownable_thing.is_owned()) {
					ownable_thing.get_owner().lose(ownable_thing);
				}
				
				// Set the owner of the object to be
				// the Person in question
				ownable_thing.set_owner(this);
				
				// Add the object to the list of the
				// Person's possessions
				possessions.add(ownable_thing);
				say("I take " + thing.name);
			}
			else {
				say("Cannot take " + thing.name);
			}
		}
		else {
			say("There is no " + thing.name + " here");
		}
	}
	
	// The opposite of take, "lose" causes a Person to drop
	// the given object. This makes the Person throw a fit. 
	public boolean lose(OwnableObject thing) {
		// The Person must be installed to lose anything
		assert is_installed();
		// The Person must already own the object in question
		if (thing.get_owner() == this) {
			assert (possessions.contains(thing));
			
			// Remove the object from the Person's possessions
			possessions.remove(thing);
			// The Person is no longer the object's owner
			thing.set_owner(null);
			
			// The Person is not happy about losing the
			// item in question
			say("I lose " + thing.toString());
			have_fit();
			return true;
		}
		else {
			say("I cannot lose what I do not have, and I have no " + thing.toString());
			return false;
		}
	}
	
	// Change the Person's restlessness
	public void set_restlessness(double value) {
		restlessness = value;
	}
	
	// All People are greetable, meaning they will be
	// greeted by other people when they move the same
	// location
	public boolean is_greetable() {
		return true;
	}
	
	// Define what a Person does in a given time step. By
	// default, the Person only moves around based on
	// his/her restlessness.
	public void tick() {
		// Make a random number from 0 to 1, if the number
		// is less than the Person's restlessness, then
		// the Person moves to a new location
		Random r = new Random();
		if (r.nextDouble() < restlessness) {
			act_randomly();
		}
	}
	
	// act_randomly causes a Person to randomly pick any
	// of the neighboring Places and move there
	public void act_randomly() {
		// Get the list of possible exits at the current
		// location
		ArrayList<String> exits = location.get_exits();
		
		Random r = new Random(); // Make a new random number generator
		if (exits != null && exits.size()>0) {
			// Randomly select one of the exits
			String randomexit = exits.get(r.nextInt(exits.size()));
			// Take that exit and move to the new Place
			go(randomexit);
		}		
	}
	
	// "move" handles moving a Person to a different location.
	// This will need to be more sophisticated than the change_location
	// function from MobileObject as we will see.
	public void move(Place loc) {
		
		// A Person can only be moved if they initially start
		// at some particular Place
		if (!is_installed()) {
			note("Cannot move until person is installed.");
			return;
		}
		System.out.println(name + " moves from " + location.name + " to " + loc.name);
		
		// Change the Person's location
		change_location(loc);
		
		// When a Person moves, so does all of his/her
		// possessions. We'll go through each one and
		// call change location on those as well
		for (int i=0; i<possessions.size(); i++) {
			possessions.get(i).change_location(loc);
		}
		
		// Now that the Person is in a new Place, the Person
		// should greet all of the people already there. To 
		// do this, we'll go through all things at the new location
		// and add to our "greetees" list, any that are greetable.
		ArrayList<SimObject> greetees = new ArrayList<SimObject>();
		for (int i=0; i<loc.get_things().size(); i++) {
			if (loc.get_things().get(i).is_greetable()) {
				greetees.add(loc.get_things().get(i));
			}
		}
		// The Person should not greet him/herself
		greetees.remove(this);
		// Greet everyone in the greetees list
		if (greetees.size() > 0) {
			for (int i=0; i<greetees.size(); i++) {
				say("Hi " + greetees.get(i).name);
			}
		}
		// Complain about the quiet if there's no one to greet
		else {
			say("It's lonely here...");
		}
	}
	
	// "go" is a lot like "move," but instead of moving to a
	// specific Place, "go" take in a direction to move in.
	// While "move" can move a Person to any Place, "go" can
	// only take a Person to a neighboring Place.
	public void go(String direction) {
		if (!is_installed()) {
			note("Cannot go until person is installed.");
			return;
		}
		// Get the Place in the specified direction
		Place newloc = location.neighbor_towards(direction);
		// Check to make sure there is a Place in that direction
		if (newloc != null) {
			move(newloc);
		}
		else {
			System.out.println("You cannot go " + direction + " from " + location.toString());
		}
	}
}
