package Resources;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;

// Places are SimObjects that represent locations in are 
// imaginary world.  Places simply consist of a list of
// things at that Place, as well as a set of adjacent Places.

@SuppressWarnings("unchecked") // Don't worry about this
public class Place extends SimObject {
	// We store the set of things at a Place as a simple ArrayList
	ArrayList<PhysicalObject> things; 
	
	// Neighbors of a Place are represented as a mapping between
	// the direction we go ("north", "south", etc.) and the Place
	// in that direction
	HashMap<String,Place> neighbors;
	
	// The constructor for Place objects 
	public Place(String name) {
		// Since Place is a SimObject we must first call
		// the constructor for SimObject
		super(name); 
		
		// Initialize the things and neighbors tables so
		// that they start empty
		this.things = new ArrayList<PhysicalObject>();
		this.neighbors = new HashMap<String,Place>();
	}
	
	public String toString() {
		return "<place: " + name + ">";
	}

	// Reset the things and neighbors lists. This isn't 
	// ever used by our code, but you might find it
	// useful for experimentation
	public void reset() {
		note("Reset");
		this.things = new ArrayList<PhysicalObject>();
		this.neighbors = new HashMap<String,Place>();
	}
	
	// Return a copy of the list of things at a Place.
	// Why a copy? We may want to change the things for
	// some extra calculations, but we don't want to 
	// alter the contents of the original list
	public ArrayList<PhysicalObject> get_things() {
		return (ArrayList<PhysicalObject>) things.clone();
	}
	
	// Check to see if an object is at a Place. Comparison
	// here is done by checking to see if the objects are
	// actually the same, not a comparison by name like
	// we do in the World class
	public boolean has_thing(PhysicalObject thing) {
		return things.contains(thing); 
	}
	
	// Add a new PhysicalObject at this Place
	public void add_thing(PhysicalObject thing) {
		things.add(thing);
	}
	
	// Remove an existing PhysicalObject at this Place
	public void remove_thing(PhysicalObject thing) {
		// We can't remove something that isn't there!
		assert has_thing(thing);
		things.remove(thing);
	}
	
	// Return a set containing the choices of what
	// direction we can move in from this Place
	// **Don't worry about what's happening in the 
	// actual code for this function
	public ArrayList<String> get_exits() {
		ArrayList<String> exits = new ArrayList<String>();
		Object[] keys = (Object[])neighbors.keySet().toArray();
		for (int i=0; i<keys.length; i++) {
			exits.add(keys[i].toString());
		}
		return exits;
	}
	
	// Given a direction, neighbor_towards returns
	// the Place in that direction.  If there isn't one
	// null is returned instead.
	public Place neighbor_towards(String direction) {
		if (neighbors.containsKey(direction)) {
			return neighbors.get(direction);
		}
		else {
			return null;
		}
	}
	
	// Add a new neighbor by specifying the Place and
	// what direction it is in from the current place.
	public void add_neighbor(String direction, Place place) {
		// Check to see that there isn't already a neighbor in
		// that direction
		if (neighbor_towards(direction)!=null) {
			throw new Error("Attempt to add duplicate neighbor to " + this.toString() + ": " + place.toString());
		}
		else {
			neighbors.put(direction, place);
		}
	}
}
