package Resources;
import java.util.HashMap;
import java.util.Scanner;

import People.Student;

public class World {
	public String name; 
	public HashMap<String,Place> places;			// "places" maps place names to their corresponding Place objects
	public HashMap<String,PhysicalObject> things;	// "things" maps thing names to their corresponding Thing objects
	
	// Constructor for World class (takes only the name of the world)
	public World(String name_) {
		name = name_;
		// Initialize the places and things maps so that they are initially empty 
		places = new HashMap<String,Place>();
		things = new HashMap<String,PhysicalObject>();
	}
	
	// Code to make our World objects print out nicely
	public String toString() {
		return "<world: " + name + ">";
	}
	
	// Given a Place object, add_place adds that place to the "places" table
	public void add_place(Place p) {
		// Check to see if the Place has already been added
		if (has_place(p.name)) {
			throw new Error("Attempt to add place with duplicate name.");
		}
		// Add the Place to the places map 
		places.put(p.name,p);
	}
	
	// Check to see if there is a Place with a certain name in the World
	public boolean has_place(String name) {
		return places.containsKey(name);
	}
	
	// Look up a Place by name. If there is no Place with that name, return null
	public Place get_place(String name) {
		if (has_place(name)) {
			return places.get(name);
		}
		else {
			return null;
		}
	}
	
	// Put an object at a certain location
	public void install_thing(PhysicalObject thing, String pname) {
		assert has_place(pname); // Check that the desired location exits in this World
		// Don't want to have too objects with the same name
		if (has_thing(thing.name)) {
			throw new Error("Attempt to add thing with duplicate name.");
		}
		// Add the object to the "things" table
		things.put(thing.name, thing);
		// Call the object's "install" method
		thing.install(get_place(pname));		
	}
	
	// Check by name to see if a PhysicalObject exists in the world
	public boolean has_thing(String name) {
		return things.containsKey(name);
	}
	// Look up a PhysicalObject by name.  If none exists with that name, return null
	public PhysicalObject get_thing(String name) {
		if (has_thing(name)) {
			return things.get(name);
		}
		return null;
	}
	
	// Connect to Places in the world with a one way connection.
	// This means a Person can move from place1 to place2, but not
	// from place2 to place1
	public void connect_one_way(String place_name1, String direction, String place_name2) {
		// Check to see that the place names refer to real places
		assert has_place(place_name1);
		assert has_place(place_name2);
		// Get the Place objects corresponding to each place name
		Place place1 = get_place(place_name1);
		Place place2 = get_place(place_name2);
		// Add place2 as a neighbor for place1 in the specified direction
		place1.add_neighbor(direction, place2);
	}
	
	// Connect to Places in the world with a two way connection.
	// This means a Person can move from both place1 to place2 and
	// back from place2 to place1 in the opposite direction.
	public void connect_both_ways(String place_name1, String direction, String place_name2) {
		connect_one_way(place_name1, direction, place_name2);
		connect_one_way(place_name2, Library.direction_opposite(direction), place_name1);
	}
	
	// Run a time step in our simulation. This amounts to simply
	// running the tick() function of every object in our World.
	public void tick() {
		// Go through every thing in the "things" list and call
		// tick() on each one
		for (int i=0; i<things.size(); i++) {
			PhysicalObject obj = Library.HashMapGet(things,i);
			obj.tick();
		}
	}
	
	// The main driver for our adventure game. It's basically a 
	// giant loop that continues to take input from the user. Based
	// on the user's input, the character performs various actions,
	// including moving, looking around, taking objects, etc.
	public void play_interactively(Person character) {
		// We use a Scanner object to read in input from the user
		Scanner s = new Scanner(System.in);
		
		// Loop forever (until the "break" command is called)
		while (true) {
			// Prompt the user
			System.out.println("What Next?");
			
			// Read in the next line of user input
			String commands = s.nextLine();
			
			// Advance decides whether or not we run a time step (tick())
			// after the user's command is run. If the user moves the
			// character, for example, this should move time forward. 
			// If the user enters a bogus command, time should not advance.
			boolean advance = false;
			
			// Separate parameters of user commands are separated by
			// spaces. Splitting on spaces will help use organize our
			// code a bit more easily.  For example, the command 
			// "get dressed" will be broken into ["get","dressed"]
			String[] cmdlist = commands.split(" ");
			
			// The main action is the first word of the command
			String action = cmdlist[0];
			
			// Remember that we have to use .equals to compare 
			// strings, we can't use == like we did in Python
			if (action.equals("quit")) {
				// Break out of the infinite loop
				System.out.println("Better luck next time.  Play again soon!");
				break;
			}
			
			// Have the character look around (counts as time step)
			else if (action.equals("look")) {
				character.look();
				advance = true;
			}
			
			// Have the character move in a given direction
			else if (action.equals("go")) {
				// A "go" command should have two values:
				// The word "go", then the direction. We
				// check that here
				if (cmdlist.length != 2) {
					System.out.println("A go command must be followed by a direction.");
					continue;
				}
				else {
					// Have the character move in the specified direction
					character.go(cmdlist[1]);
					advance = true;
				}
			}
			
			// Have the character take an ownable object at his/her
			// current location (referenced by name)
			else if (action.equals("take")) {
				// A "take" command should have two values:
				// The word "take", then the name of the object
				// to take.
				if (cmdlist.length != 2) {
					System.out.println("A take command must be followed by the name of the item you want to take");
					continue;
				}
				else {
					// Look up the OwnableObject by name using get_thing,
					// then have the character take that object
					character.take(get_thing(cmdlist[1]));
					advance = true;
				}
			}
			
			// Special commands used for students to get 
			// dressed an undressed
			else if (action.equals("get")) {
				// Check that the user provides both the word
				// "get" and a second input specifying either
				// "dressed" or "undressed"
				if (cmdlist.length != 2) {
					System.out.println("A get command must be followed by a second attribute.");
				}
				else if (cmdlist[1].equals("dressed")) {
					// Check to see that character is a student.
					// Only Students can get dressed and undressed
					if ((character instanceof Student)) {						
						// Since the character object is of type "Person"
						// and Person objects do not have a get_dressed() 
						// method, we need to first CAST the character to 
						// the Student type. In other words, we're telling
						// Java, we're really sure that character is a Student
						// so it's fine to call Student methods on it
						((Student) character).get_dressed();
						advance = true;
					}
					else {
						System.out.println("Only students can get dressed an undressed!");
					}
				}
				else if (cmdlist[1].equals("undressed")) {
					// Check that character is a student
					if ((character instanceof Student)) {
						((Student) character).get_undressed();
						advance = true;
					}
					else {
						System.out.println("Only students can get dressed an undressed!");
					}
				}
				// Print error message if user types "get blahblah"
				// or something like that
				else {
					System.out.println("Sorry, I don't know how to get " + cmdlist[1]);
				}
			}
			// Print error message if the command is not recognized
			else {
				System.out.println("Sorry, I don't know how to " + action); 
			}
			// If advance is true, we move forward a time step
			// by calling tick()
			if (advance) {
				tick();
			}
		}
		// Stop trying to read in user input 
		s.close();
	}
}
